-- F-Convert
--    Version: 1.0
--       by Robert Szalay

-- possible command line options(only one can be used at a time)
-- -w  convert to windows/dos format
-- -d  convert to windows/dos format
-- -u  convert to unix format
-- -m  convert to mac format
-- -c  just check the format

include file.e
include get.e



sequence types, sep

types = {"Windows/DOS","Unix","Macintosh"}
sep = {"\r\n","\n","\r"}

atom  Handle,     -- Handle for reading/writing to the file
      commandlen, -- length of commandline
      ftype,      -- the file type of the file
      fto,        -- the file type to convert to
      chars,      -- number of characters in the file
      lines       -- number of lines in the file

sequence fbuf,          -- buffer for the file
         commandline,   -- the command line
         infile,        -- file to be read
         outfile        -- file to be written

fto = 0
chars = 0
lines = 0
fbuf = ""
commandline = command_line()
commandlen = length(commandline)

function tokenize(sequence s, atom c)
sequence tmp
atom a, pos, len
tmp = {}
a = find(c,s)
pos = 0
len = length(s)
while a do
   tmp = append(tmp,s[1..a-1])
   s = s[a+1..len]
   len-=a
   a = find(c,s)
end while
if len then
   tmp = append(tmp,s)
end if
return tmp
end function


function flatten(sequence s)
sequence t t = ""
for i = 1 to length(s) do
   t&=s[i]
end for
return t
end function


function open_file()
   return open(infile,"rb")
end function


function check_format()
object junk
atom l
junk = gets(Handle)
if seek(Handle,0) then end if -- return to beginning of file
if atom(junk) then
   return -1
end if
l = length(junk)
if junk[l] = '\n' then
   if junk[l-1] = '\r' then
      return 1
   else
      return 2
   end if
else
   if find('\r',junk) then
      return 3
   end if
end if
return 0
end function


procedure file_in()
object junk
integer tlen
fbuf = ""
tlen = length(sep[ftype])
while 1 do
   junk = gets(Handle)
   if atom(junk) then
      exit
   end if
   if ftype != 3 then
      lines+=1
      fbuf = append(fbuf,junk[1..length(junk)-tlen])
   else
      fbuf &= junk
   end if
end while
if ftype = 3 then
   fbuf = tokenize(fbuf,'\r')
   lines = length(fbuf)
end if
close(Handle)
end procedure


procedure convert()
for i = 1 to lines do
   fbuf[i] &= sep[fto]
end for
end procedure


procedure file_out()
Handle = open(outfile,"wb")
if Handle != -1 then
   puts(Handle,fbuf)
   close(Handle)
else
   puts(1,"Could not open "&outfile&" for writing.\n")
   abort(1)
end if
end procedure


procedure show_info()
   puts(1,"F-Convert\n\tVersion: 1.0\n\t\tby Robert Szalay\n\n")
   puts(1,"Usage: ex fconvert.ex infile <-w|-d|-u|-m|-c> [outfile]\n")
   puts(1,"  -w\tConvert to Windows/DOS format\n")
   puts(1,"  -d\tConvert to Windows/DOS format\n")
   puts(1,"  -u\tConvert to Unix format\n")
   puts(1,"  -m\tConvert to Mac format\n")
   puts(1,"  -c\tCheck the file format(without converting it)\n")
   puts(1,"You must specify one and only one of these options.\n\n")
   puts(1,"If the filename contains any spaces, enclose it in \"quotes\".\n\n")
   puts(1,"outfile is an optional parameter.\nIf it is not used, the output will go into infile.\n\n")
   puts(1,"Example1: ex fconvert myfile.txt -m myfile.mac.txt\n")
   puts(1,"Example2: ex fconvert \"The File.txt\" -m \"The Converted File.txt\"\n")
   puts(1,"Example3: ex fconvert \"Another File.txt\" -m\n\n")
end procedure


procedure read_cmd()
if commandlen < 3 then
   show_info()
   abort(1)
end if
infile = commandline[3]
if commandlen > 3 then
   if find(commandline[4],{"-w","-d","-W","-D"}) then
      fto = 1
   elsif find(commandline[4],{"-u","-U"}) then
      fto = 2
   elsif find(commandline[4],{"-m","-M"}) then
      fto = 3
   elsif find(commandline[4],{"-c","-C"}) then
      fto = 4
   end if
end if
if commandlen = 5 then
   outfile = commandline[5]
else
   outfile = infile
end if
end procedure


read_cmd()


Handle = open_file()

if Handle = -1 then -- file not found
   puts(1,"File not found.\n")
   abort(1)
end if

ftype = check_format()
if ftype < 1 then
   if ftype = 0 then
      puts(1,"Unknown format.\n")
      abort(1)
   elsif ftype = -1 then
      puts(1,"File is empty.\n")
      abort(1)
   end if
end if

if ftype = fto then
   puts(1,infile&" is already in "&types[fto]&" format.\n")
   abort(1)
end if

if fto > 0 and fto < 4 then
   puts(1,"Reading file...\n")
   file_in()
   puts(1,"Converting from "&types[ftype]&" to "&types[fto]&" format...\n")
   convert()
   fbuf = flatten(fbuf)
   chars = length(fbuf)
   puts(1,"Writing converted file to "&outfile&"...\n")
   file_out()
   puts(1,"Operation complete.\n")
   printf(1,"%s is %d lines and %d characters.\n",{outfile,lines,chars})
elsif fto = 4 then
   puts(1,"File type is "&types[ftype]&"\n")
else
   puts(1,"Required parameter missing.\n")
   show_info()
end if
--convert("F:\\fileconvert\\ex.err",1
